@@ -30,7 +30,7 @@ int unrec = 1;
 int tasks = TASKS;
 char scantype = 't';
 char *only_prot = NULL;
-int mode;
+int amap_mode;
 
 // for banner printing
 unsigned char banner_string[256];
@@ -63,7 +63,7 @@ Options:
     -d        Dump hex traffic (only if a response is received)
     -b        Print ascii banner of responses
     -T CONS   Amount of parallel connections to make (default %d, max %d)
-    -p PROTO  Only test for applicational protocol PROTO (i.e.: ftp)
+    -p PROTO  Only send the trigger for applicational protocol PROTO (i.e. ftp)
     -t SEC    Response timeout, wait longer on slow connections (default %d)
     -D FILE   Read from Definitions FILE[.trig|.resp|.rpc] instead of default
     -h        Print this shit
@@ -348,7 +348,8 @@ FILE *open_amap_file(char *fnam, char *type, char *extension)
   int i = 0;
 
   if (fnam != NULL) {
-    strcpy(file_name, fnam);
+    strncpy(file_name, fnam, sizeof(file_name)-10);
+    file_name[sizeof(file_name)-10] = 0;
     strcat(file_name, extension);
     f = fopen(file_name, "r");
   } else
@@ -531,7 +532,7 @@ char *response_index(char *str)
 ll_resp *read_responses(char *fnam)
 {
   ll_resp *first_resp = NULL, *curr_resp = NULL;
-  ll_response *curr_response, *new_response = NULL;
+  ll_response *curr_response = NULL, *new_response = NULL;
   FILE *resp_file = NULL;
   char *in_line;
   char *readln = NULL, *walk = NULL, *walk2;
@@ -922,6 +923,7 @@ int compile_line(char *trig, unsigned char *out_bytes, int type, char proto)
     temp2[19] = 74;
     return (40 + j);
     break;
+
   case TRIG_STR:
     strcpy(temp1, trig);
     temp2 = rindex(temp1, '"');
@@ -1005,10 +1007,17 @@ int amap_conn(sock_block * babble)
 
   my_tar.sin_port = htons((unsigned short int) babble->port);
   my_tar.sin_family = AF_INET;
+#ifdef CYGWIN
+  if (inet_aton(babble->ip_ad, &addie) <= 0) {
+    warn("inet_aton() failed");
+    return (-9999);
+  }
+#else
   if (inet_pton(AF_INET, babble->ip_ad, &addie) <= 0) {
     warn("inet_pton() failed");
     return (-9999);
   }
+#endif
   my_tar.sin_addr.s_addr = addie.s_addr;
 
   ret = -1;
@@ -1023,7 +1032,9 @@ int amap_conn(sock_block * babble)
 	if (SSL_connect(ssl) < 0) {
 	  babble->ssl_enabled = 0;
 	  // dump this connection and reconnect
-	  close(s);
+//	  close(s); // XXX test
+	  shutdown(s, SHUT_RDWR); // XXX new
+	  s = -1;
 	  ret = -1;
 	}
       }
@@ -1032,10 +1043,12 @@ int amap_conn(sock_block * babble)
     if (ret < 0) {
       fail++;
       if (fail >= MAX_RETRIES) {
-	close(s);
+//	close(s); // XXX test
+	shutdown(s, SHUT_RDWR); // XXX new
+	s = -1;
 	return -1;
       }
-      sleep(1);
+      usleep(250000);
     }
   }
 
@@ -1223,7 +1236,7 @@ void lookup_response(sock_block * babble, FILE * outp_file,
 //  char *str1, *str2;
   int gogo;
 
-  if (mode != SCAN_RPC) {
+  if (amap_mode != SCAN_RPC) {
     babble->orig_response = malloc(babble->response_length);
     memcpy(babble->orig_response, babble->response, babble->response_length);
     orig_response = babble->orig_response;
@@ -1277,7 +1290,7 @@ void lookup_response(sock_block * babble, FILE * outp_file,
 	       || strncasecmp(my_resp->prot, "rpc-", 4) == 0))
 	    mapped_port->rpc = 1;
 
-	  if (mode == SCAN_PLAIN && mapped_port->ip_prot == 't'
+	  if (amap_mode == SCAN_PLAIN && mapped_port->ip_prot == 't'
 	      && (strcasecmp(my_resp->prot, "ssl") == 0
 		  || strncasecmp(my_resp->prot, "ssl-", 4) == 0))
 	    mapped_port->ssl = 1;
@@ -1422,10 +1435,10 @@ void read_response(sock_block * babble, ll_targets * s_target,
     memset(rline, 0, sizeof(rline));
     rlen = -1;
 
-    if (babble[i].active == 1) {	// OK, have something for us yet?
+    if (babble[i].active == 1 && babble[i].s != -1) {	// OK, have something for us yet?
       // if we have a few tasks only, fast networks will fuck us up
       if (tasks < TASKS)
-	sleep(1);
+	usleep(250000);
       errno = 0;
       if (babble[i].ssl_enabled) {
 	rlen = SSL_read(babble[i].ssl_sock, rline, sizeof(rline));
@@ -1441,9 +1454,10 @@ void read_response(sock_block * babble, ll_targets * s_target,
 	// Lookup the response and print
 	lookup_response(&(babble[i]), outp_file, s_target);
 	// OK, this socket is done, free it up for new use.
-	close(babble[i].s);
-	running_tasks--;
+//	close(babble[i].s); // XXX
+	shutdown(babble[i].s, SHUT_RDWR); // XXX new - test!
 	clear_block(&(babble[i]));
+	running_tasks--;
       } else {
 	// check to see it this socket hasn't outlived it's poor little life
 	if (errno == ECONNREFUSED && babble[i].prot == 'u') {
@@ -1463,7 +1477,8 @@ void read_response(sock_block * babble, ll_targets * s_target,
 		   port_todo->ip_prot == 't' ? "tcp" : "udp",
 		   port_todo->port, targ_todo->target);
 	  running_tasks--;
-	  close(babble[i].s);
+//	  close(babble[i].s); // XXX test
+	  shutdown(babble[i].s, SHUT_RDWR); // XXX new!
 	  clear_block(&(babble[i]));
 	}
       }
@@ -1487,7 +1502,7 @@ int do_scan(int scanmode, FILE * outp_file, ll_targets * s_target,
   int ready_for_next;
   char compiled_t[BUFSIZE];
 
-  mode = scanmode;
+  amap_mode = scanmode;
 
 /* initialise all socket blocks */
   for (i = 0; i < MAX_TASKS; i++)
@@ -1506,9 +1521,9 @@ int do_scan(int scanmode, FILE * outp_file, ll_targets * s_target,
 	printf("%d ", port_todo->port);
       trigger_todo = s_trigger;
       while (trigger_todo != NULL
-	     && (mode == SCAN_PLAIN ||
-		 (mode == SCAN_SSL && port_todo->ssl) ||
-		 (mode == SCAN_RPC && port_todo->rpc)
+	     && (amap_mode == SCAN_PLAIN ||
+		 (amap_mode == SCAN_SSL && port_todo->ssl) ||
+		 (amap_mode == SCAN_RPC && port_todo->rpc)
 	     )
 	) {
 	/* now see if this task is appropriate */
@@ -1519,8 +1534,8 @@ int do_scan(int scanmode, FILE * outp_file, ll_targets * s_target,
 	  total_tasks++;
 	trigger_todo = (ll_triggers *) trigger_todo->next;
       }
-      if ((mode == SCAN_SSL && port_todo->ssl)
-	  || (mode == SCAN_RPC && port_todo->rpc)) {
+      if ((amap_mode == SCAN_SSL && port_todo->ssl)
+	  || (amap_mode == SCAN_RPC && port_todo->rpc)) {
 	strcpy(port_todo->appl_ids->appl_id, "unidentified");
 	port_todo->appl_ids->next = NULL;
       }
@@ -1533,7 +1548,7 @@ int do_scan(int scanmode, FILE * outp_file, ll_targets * s_target,
 
   if (verbose) {
     printf("Total amount of tasks to perform in ");
-    switch (mode) {
+    switch (amap_mode) {
     case SCAN_PLAIN:
       printf("plain connect");
       break;
@@ -1553,6 +1568,11 @@ int do_scan(int scanmode, FILE * outp_file, ll_targets * s_target,
   if (total_tasks < tasks)
     tasks = total_tasks;
 
+  if (amap_mode == SCAN_RPC) {
+    one_is_enough = 1; // force one_is_enough identification for RPC
+//    printf("Starting RPC identification - this can take looong time. Press Control-C if you\nare already satisfied with the results. Disable RPC identification with -R.\n");
+  }
+
 /* Here comes the parallelising using non-blocking sockets */
 
 /* again, loop through the tasks, but now not for counting, but for
@@ -1567,16 +1587,16 @@ int do_scan(int scanmode, FILE * outp_file, ll_targets * s_target,
       while (port_todo != NULL) {
 	/* now see if this task is appropriate */
 	if (port_todo->skip != 1 &&
-	    (mode == SCAN_PLAIN ||
-	     (mode == SCAN_SSL && port_todo->ssl) ||
-	     (mode == SCAN_RPC && port_todo->rpc)
+	    (amap_mode == SCAN_PLAIN ||
+	     (amap_mode == SCAN_SSL && port_todo->ssl) ||
+	     (amap_mode == SCAN_RPC && port_todo->rpc)
 	    ) &&
 	    (one_is_enough == 0
 	     || strcmp(port_todo->appl_ids->appl_id, "unidentified") == 0)
 	    &&
-	    (!((trigger_todo->ip_prot == 't') && (port_todo->ip_prot == 'u'))
+	    (!(((trigger_todo->ip_prot == 't') && (port_todo->ip_prot == 'u'))
 	     || ((trigger_todo->ip_prot == 'u')
-		 && (port_todo->ip_prot == 't')))) {
+		 && (port_todo->ip_prot == 't'))))) {
 	  /* Main loop */
 	  // Check for available sock_block
 	  ready_for_next = 0;
@@ -1606,7 +1626,7 @@ int do_scan(int scanmode, FILE * outp_file, ll_targets * s_target,
 	  babble[i].port_ptr = (ll_ports *) port_todo;
 	  babble[i].target_ptr = (ll_targets *) targ_todo;
 
-	  if (mode != SCAN_PLAIN && port_todo->ssl)
+	  if (amap_mode != SCAN_PLAIN && port_todo->ssl)
 	    babble[i].ssl_enabled = 1;
 
 	  switch (amap_conn(&(babble[i]))) {
@@ -1642,6 +1662,13 @@ int do_scan(int scanmode, FILE * outp_file, ll_targets * s_target,
     }
     trigger_todo = (ll_triggers *) trigger_todo->next;
   }
+  
+  for (i = 0; i < MAX_TASKS; i++)
+    if (babble[i].s != -1) {
+//	close(babble[i].s); // XXX
+	shutdown(babble[i].s, SHUT_RDWR); // XXX new - test!
+	clear_block(&(babble[i]));
+    }
 
   if (verbose)
     printf("%d responses received in total for %d tasks.\n", resp_recv,
@@ -1770,7 +1797,9 @@ int main(int argc, char *argv[])
   ll_ports *first_port;
   ll_triggers *first_trig, *rpc_trig = NULL;
   struct in_addr in;
+#ifndef CYGWIN
   char out[16];
+#endif
 
   char *nmap_infile = NULL, *output_file = NULL;	// otherwise logfiles with weird filenames will be created ...
   char *deffile = NULL;
@@ -1859,9 +1888,12 @@ int main(int argc, char *argv[])
       if ((target = gethostbyname(argv[optind])) == NULL)
 	exit_err("Error resolving target, ever heard of DNS and IP?\n");
       memcpy(&in, target->h_addr, target->h_length);
-      //argv[optind] = inet_ntoa(in); //deprecated
+#ifdef CYGWIN
+      argv[optind] = inet_ntoa(in); //deprecated
+#else
       argv[optind] =
 	(char *) inet_ntop(AF_INET, &in, (char *) &out, sizeof(out));
+#endif
     }
     if ((first_targ = (ll_targets *) malloc(sizeof(ll_targets))) == NULL)
       exit_err("Malloc failed\n");
