@@ -1,5 +1,5 @@
 /*
- * AmapCrap v2.5 (c) 2002 by van Hauser / THC <vh@thc.org>
+ * AmapCrap v2.7 (c) 2003 by van Hauser / THC <vh@thc.org>
  * http://www.thc.org
  *
  * Sends random data to silent ports to illicit a reponse. For use with amap
@@ -53,7 +53,7 @@ int warn = 0;
 
 void help()
 {
-  printf("amapcrap v2.5 (c) 2003 by van Hauser /THC <vh@thc.org>
+  printf("amapcrap v2.7 (c) 2003 by van Hauser /THC <vh@thc.org>
 
 Syntax: %s [-S] [-u] [-m 0ab] [-M min,max] [-n connects] [-N delay] [-w delay] [-e] [-v] TARGET PORT
 
@@ -82,7 +82,7 @@ int main(int argc, char *argv[])
 {
   unsigned short int port = 0;
   long int max_connects = UNLIMITED;
-  int sent_mode, sent_mode_old;
+  int sent_mode = 0, sent_mode_old = 0;
   int send_nullbytes = 1;
   int send_ascii = 1;
   int send_binary = 1;
@@ -99,7 +99,7 @@ int main(int argc, char *argv[])
   int i, j;
   int s;
   int ret;
-  int reads;
+  int reads = 0;
   int sock_type = SOCK_STREAM;
   int sock_protocol = IPPROTO_TCP;
   unsigned char buf[8196];
@@ -107,6 +107,9 @@ int main(int argc, char *argv[])
   struct sockaddr_in target;
   struct hostent *resolv;
   int res = 0;
+#ifndef CYGWIN
+  char out[16];
+#endif
 
   srand((getpid() + getuid() + getgid()) ^ time(0));
 
@@ -235,7 +238,11 @@ int main(int argc, char *argv[])
     close(i);
 
   printf("# Starting AmapCrap on %s port %d\n",
+#ifdef CYGWIN
 	 inet_ntoa((struct in_addr) target.sin_addr), port);
+#else
+         inet_ntop(AF_INET, &target.sin_addr, (char *) &out, sizeof(out)), port);
+#endif
   (void) setvbuf(stdout, NULL, _IONBF, 0);
   printf("# Writing a \"+\" for every 10 connect attempts\n# ");
 
@@ -464,7 +471,7 @@ int main(int argc, char *argv[])
 // output function
 	  printf("\n\n# Put this line into appdefs.trig:\n");
 	  if (sent_mode == 1) {
-	    printf("PROTOCOL_NAME:%c::\"",
+	    printf("PROTOCOL_NAME:%c:0:\"",
 		   sock_type == SOCK_DGRAM ? 'u' : 't');
 	    for (i = 0; i < str_len; i++)
 	      switch (str[i]) {
@@ -482,7 +489,7 @@ int main(int argc, char *argv[])
 	      }
 	    printf("\"");
 	  } else {
-	    printf("PROTOCOL_NAME:%c::0x",
+	    printf("PROTOCOL_NAME:%c:0:0x",
 		   sock_type == SOCK_DGRAM ? 'u' : 't');
 	    for (i = 0; i < str_len; i++) {
 	      printf("%c",
